function [summary result] = Compute_MLL(y,nb_MCMC,path_dep,type_model,prior_alpha_kappa_a_b,prior_rho_a_b,prior_eta_a_b,summary,result)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% The Compute_MLL function estimates the MLL of the IHMM-GARCH model (or
%%% the IHMM-HMP model if path_dep = 0) for the time series y. As it relies
%%% on a sequential applications of MCMC algorithm, you have to specify the
%%% number of MCMC draws (after convergence) that you want to use for
%%% computing the predictive densities. The ESS function for selecting h on
%%% the fly is implemented (see paper).
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Input : 
%%% y : a Univariate time series.
%%% nb_MCMC : the number of MCMC iteration after convergence.
%%% path_dep : if it is equal to zero, the estimated model is an IHMM-HMP. Otherwise, IHMM-GARCH.
%%% prior_alpha_kappa_a,prior_rho_a,prior_rho_b : stand for testing the
%%% prior sensitivity. Default values are [1 10], [10 1], [1 10] (see Table on the
%%% prior distribution in the paper).
%%% No need to specify summary and result.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Output : the structure result contains many fields : 
%%%%% summary : is a structure that contains all the predictive MLL from
%%% time=1 until T and the ESS criterion for each time.
%%%%% result : is a structure containing summaries of the last posterior
%%%%% distribution computed by the MCMC algorithm. We refer to the function
%%%%% IHMS_GARCH for more details on the structure.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if(nargin<9)
    summary = 0;
    result = 0;
    if(nargin<7)
        prior_eta_a_b = [1 10];
        if(nargin<6)
            prior_rho_a_b = [10 1];
            if(nargin<5)
                prior_alpha_kappa_a_b = [10 1];
                if(nargin<4)
                    type_model = 'KL';
                    if(nargin<3)
                        path_dep = 1;
                    end
                end
            end
        end
    end
end
nb_saut_dep = 15;
nb_saut = 100;
taille = max(size(y));



if(isstruct(summary) && isstruct(result))
    ESS = summary.ESS;
    MLL = summary.MLL;
    count = summary.count;
    nb_saut = summary.nb_saut;
    nb_MCMC_current = summary.nb_MCMC_current;
    path_dep = summary.path_dep;
    type_model = summary.type_model;
    prior_alpha_kappa_a_b = summary.prior_alpha_kappa_a_b;
    prior_rho_a_b = summary.prior_rho_a_b;
    prior_eta_a_b = summary.prior_eta_a_b;
    GARCH = 0;
else
    
    MLL = zeros(taille,1);
    ESS = zeros(taille,1);
    
    
    
    
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%%%%%%%%% Calcul du premier MLL predictif sans avoir de donnes
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    nb_rep = 1e4;
    hyper_mean_mu = [log(0.5/4.5) log(0.15/0.85) log(0.7/0.8)];
    hyper_var_mu = eye(3);
    log_mu = mvnrnd(hyper_mean_mu,hyper_var_mu,nb_rep);
    
    df_wish = 50;
    inv_V = (5*df_wish)*eye(3);
    GARCH = zeros(nb_rep,3);
    log_GARCH = zeros(nb_rep,3);
    sigma = zeros(nb_saut_dep,nb_rep);
    like = zeros(nb_saut_dep,nb_rep);
    unc_var = var(y(1:nb_saut_dep));
    y_sq = y(1:nb_saut_dep).*y(1:nb_saut_dep);
    for i=1:nb_rep
        Sigma = iwishrnd(inv_V,df_wish);
        log_GARCH(i,:) = mvnrnd(log_mu(i,:),Sigma);
        GARCH(i,:) = [5*exp(log_GARCH(i,1))/(1+exp(log_GARCH(i,1)))  (exp(log_GARCH(i,2)))/(exp(log_GARCH(i,2))+1)  1.5*exp(log_GARCH(i,3))/(1+exp(log_GARCH(i,3)))];
        [sigma(:,i) like(:,i)] = log_likelihood(y_sq,GARCH(i,:),unc_var);
    end
    
    like = cumsum(like);
    for q=1:nb_saut_dep
        maxi = max(like(q,:));
        dens = exp(like(q,:)-maxi);
        
        MLL(q) = log(mean(dens)) + maxi - sum(MLL(1:q-1));
        w = dens/sum(dens);
        ESS(q) = 1/(nb_rep*sum(w.*w));
    end
    summary.GARCH = GARCH;
    nb_MCMC_current = nb_MCMC;
    count = nb_saut_dep;
end
redo_same = 0;
% for i=1:3
%    [f,x] = ksdensity(GARCH(:,i));
%    subplot(1,3,i),plot(x,f);
% end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%% Calcul des MLL predictifs %%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
cd IHMS_GARCH_cpp;
system('erase Griddy_res*.dat');
system('erase data_matlab.dat');
cd ..;

while(count<taille)
    if(count+nb_saut<=taille)
        nb_forecast = nb_saut;
    else
        nb_forecast = taille-count;
    end
    count = count + nb_forecast;
    if(count>nb_saut+nb_saut_dep)
        [~,max_prob_regime] = max(result.prob_nb_regime);
        sn = result.sn_mode(max_prob_regime,:);
        etat = sn(end);
        sn(end+1:count-nb_forecast) = etat;
                
        cd IHMS_GARCH_cpp;
        save_file(sn,['Griddy_res_start_data_matlab_MS_' num2str(max_prob_regime) '.dat'],'.\');
        cd ..;
        
        summary.ESS = ESS;
        summary.MLL = MLL;
        summary.count = count-nb_forecast;
        summary.nb_saut = nb_saut;
        summary.nb_MCMC_current = nb_MCMC_current;
        summary.nb_forecast = nb_forecast;
        summary.path_dep = path_dep;
        summary.type_model = type_model;
        summary.max_prob_regime = max_prob_regime;
        summary.prior_alpha_kappa_a_b = prior_alpha_kappa_a_b;
        summary.prior_rho_a_b = prior_rho_a_b;
        summary.prior_eta_a_b = prior_eta_a_b;
        save('current_MLL.mat','summary','result');
        [result] = IHMS_GARCH(y(1:count),nb_MCMC_current,nb_forecast,path_dep,type_model,max_prob_regime,prior_eta_a_b,prior_rho_a_b,prior_alpha_kappa_a_b);
    else
        [result] = IHMS_GARCH(y(1:count),nb_MCMC_current,nb_forecast,path_dep,type_model,1,prior_eta_a_b,prior_rho_a_b,prior_alpha_kappa_a_b);
    end  
    close all;
    
%     taille_LPD = size(result.LPD);
%     ind = find(result.LPD(:,end)>-1000);
%     dim_LPD = sum(max(size(ind)));
%     if(dim_LPD~=taille_LPD(1))
%         LPD = result.LPD(dim_LPD,:);
%     else
%         LPD = result.LPD;
%     end
    LPD = result.LPD;
    LPD = cumsum(LPD')';
    dim_LPD = max(size(LPD));
    for q=1:nb_forecast
        maxi = max(LPD(:,q));
        dens = exp(LPD(:,q)-maxi);
        
        MLL(count-nb_forecast+q) = log(mean(dens)) + maxi - sum(MLL(count-nb_forecast+1:count-nb_forecast+q-1));
        w = dens/sum(dens);
        ESS(count-nb_forecast+q) = 1/(dim_LPD*sum(w.*w));
    end

    if(ESS(count-nb_forecast+1)<0.1 && redo_same==0)
        redo_same = 1;
        nb_MCMC_current = 2*nb_MCMC;
        it = 1;
    else
        nb_MCMC_current = nb_MCMC;
        redo_same = 0;
        it = 2;
        test = 1;
        while(test==1 && it<=nb_forecast)
            if(ESS(count-nb_forecast+it)<0.1)
                test = 0;
            else
                it = it+1;
            end
        end
    end
    
    nb_saut_ESS = it-1;
    count = count - nb_forecast + nb_saut_ESS;
    disp(['ITERATION : ' num2str(count)])
    
    
end


summary.ESS = ESS;
summary.MLL = MLL;
summary.GARCH = GARCH;