function [result] = IHMS_GARCH(y,nb_MCMC,forecast,path_dep,type_model,regime,prior_alpha_kappa_a_b,prior_rho_a_b,prior_eta_a_b)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% The IHMS_GARCH function draws the posterior distribution of a
%%% IHMM-GARCH model. The number of regimes is included in the MCMC
%%% algorithm. For more information, we refer to the paper 'Infinite-State
%%% Markov-Switching for Dynamic Volatility'.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Input : 
%%% y : a Univariate time series.
%%% nb_MCMC : the number of MCMC iteration after convergence.
%%% forecast : if it is equal to x, the algorithm forecasts the TS on the
%%% x last observations and provides the MSE and the LPD.
%%% Path_dep : if it is equal to zero, the model is an IHMM-GARCH of Haas,
%%% Mittnik and Paolella. Otherwise, IHMM-GARCH (with path dependence).
%%% Regime : denotes the number of regime for starting the MCMC (default :
%%% 1)
%%% prior_alpha_kappa_a,prior_rho_a,prior_rho_b : stand for testing the
%%% prior sensitivity. Default values are [1 10], [10 1], [1 10] (see Table on the
%%% prior distribution in the paper).
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Output : the structure result contains many fields : 
%%% regime_max : all the number of regimes observed during the MCMC.
%%% omega, alpha , beta : MCMC draws of omega, alpha and beta parameters.
%%% p : MCMC draws of the transition matrix.
%%% alp_kap_eta : MCMC draws of the IHMM parameters.
%%% sn : MCMC draws of the state vector (type in int8).
%%% prob_sn : posterior probabilities of being in one regime over time.
%%% isCP : posterior probabilities of being in a Change-point regime.
%%% (non-recurrent regime) over time.
%%% prob_cass : posterior probabilities of having a break of time.
%%% post_dens : MCMC densities.
%%% mean_omega_time : posterior mean of omega over time.
%%% mean_alpha_time : posterior mean of alpha over time.
%%% mean_beta_time : posterior mean of beta over time.
%%% mean_unc_var : posterior mean of the unconditional variance over time.
%%% mean_sigma : posterior mean of the conditional variance over time.
%%% prob_nb_regime : posterior probabilities of having a specific number of
%%% regime.
%%% mode_theta : posterior means of the GARCH parameters over time for the
%%% highest likely number of regime.
%%% mean_theta : posterior means of the GARCH parameters over time.
%%% std_omega_time : posterior std of the omega parameter over time.
%%% std_alpha_time : posterior std of the alpha parameter over time.
%%% std_beta_time : posterior std of the beta parameter over time.
%%% mode_sn : posterior mode of the state vector for the different observed
%%% number of regimes.
%%% unc_var_mode : unconditional variance over time at the mode parameter.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% Example : [result] = IHMS_GARCH(y,10000);
%%% The posterior distribution of the parameters given the data y will
%%% contain 10000 draws after convergence. The IHMM-GARCH model will be
%%% estimated (Path_dep=1) and without forecasting the series (forecast=0)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% Extra choices for the MCMC algorithm
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
debug = 0; %% If = 1, to debug the program
do_convergence = 1; %% If = 0, no burn-in and no test of being in the stationary distribution
BPR = 0; %% If = 1, estimation using the BPR approach (with care because the IHMM structure does not work with it)
sampling_Kl_HMP = 1; %% If the sampling of the state vector is done by Klaassen or HMP
CP = 0; %% If the estimation is a CP or a MS
k_beam = 0.1; %% Value between 0 and 1 related to the truncated uniform distribution of the paper
fix_eta = 0; %% If = x>0,  eta is fixed to x in the MCMC sampler
fix_rho = 0; %% if = x in [0,1], rho is set to x in the MCMC sampler
fix_alpha_kappa = 0; %% if = x>0, alpha + kappa is fixed to x in the MCMC sampler
Ros_by_regime = 1; %% Related to the proposal distribution of the GARCH parameters
IHMM_sample = 1; %% If = 1, the IHMM structure is on otherwise, estimation of a MS/CP GARCH with fixed regimes determined by the imput regime


if(nargin<9)
    prior_eta_a_b = [1 10]; %% hyper-parameters of eta
    if(nargin<8)
        prior_rho_a_b = [10 1]; %% hyper-parameters of rho
        if(nargin<7)
            prior_alpha_kappa_a_b = [1 10]; %% hyper-parameters of alpha + kappa
            if(nargin<6)
                regime = 1;
                if(nargin<5)
                    type_model = 'kl';
                    if(nargin<4)
                        path_dep = 1;
                        if(nargin<3)
                            forecast = 0;
                        end
                    end
                end
            end
        end
    end
end
                                                    

switch type_model
    case 'HMP'
        Haas = 1;
    otherwise
        Haas = 0;
end
if(IHMM_sample==0)
    forecast = 0;
end
if(size(y,1)<size(y,2))
    y = y';
end

cd IHMS_GARCH_cpp;
save_file(y,'data_matlab.dat','.\');
delete('.\Griddy_res_start_data_matlab*.dat');
name = ['IHMM_GARCH ' num2str(regime) ' ' num2str(nb_MCMC) ' 65 7 1 50 2 0 0.5 0 ' num2str(regime) ' ' num2str(CP) ' ' num2str(path_dep) ' ' num2str(Haas) ' ' ...
    num2str(forecast) ' ' num2str(prior_alpha_kappa_a_b(1)) ' ' num2str(prior_alpha_kappa_a_b(2)) ' ' num2str(prior_rho_a_b(1)) ' '...
    num2str(prior_rho_a_b(2)) ' ' num2str(prior_eta_a_b(1)) ' ' num2str(prior_eta_a_b(2)) ' ' num2str(IHMM_sample) ' ' ...
    num2str(Ros_by_regime) ' ' num2str(fix_alpha_kappa) ' ' num2str(fix_rho) ' ' num2str(fix_eta) ' ' num2str(k_beam) ' ' ...
    num2str(sampling_Kl_HMP) ' ' num2str(BPR) ' ' num2str(do_convergence) ' ' num2str(debug) ' data_matlab'];

system(name);
[result]=Read_param_IHMM(y,1,'.\',forecast,nb_MCMC);
system('erase Griddy_res*.dat');
system('erase data_matlab.dat');
cd ..;
result.prior_alpha_kappa_a_b = prior_alpha_kappa_a_b;
result.prior_rho_a_b = prior_rho_a_b;
result.prior_eta_a_b = prior_eta_a_b;



